@@ -17,6 +17,7 @@ from collections import namedtuple
 # Project
 from ..utils.compat.misc import override__dir__
 from ..extern import six
+from ..utils.compat.odict import OrderedDict
 from ..utils.exceptions import AstropyDeprecationWarning
 from .. import units as u
 from ..utils import OrderedDict
@@ -144,10 +145,21 @@ class FrameAttribute(object):
         A new data descriptor to hold a frame attribute
     """
 
+    _nextid = 1
+    """
+    Used to ascribe some ordering to FrameAttribute instances so that the
+    order they were assigned in a class body can be determined.
+    """
+
     def __init__(self, default=None, secondary_attribute=''):
         self.default = default
         self.secondary_attribute = secondary_attribute
 
+        # Use FrameAttribute._nextid explicitly so that subclasses of
+        # FrameAttribute use the same counter
+        self._order = FrameAttribute._nextid
+        FrameAttribute._nextid += 1
+
     def convert_input(self, value):
         """
         Validate the input ``value`` and convert to expected attribute class.
@@ -489,12 +501,20 @@ class BaseCoordinateFrame(object):
 
     @classmethod
     def get_frame_attr_names(cls):
-        out = {}
+        seen = set()
+        attributes = []
         for mro_cls in cls.__mro__:
             for name, val in mro_cls.__dict__.items():
-                if issubclass(val.__class__, FrameAttribute) and name not in out:
-                    out[name] = getattr(mro_cls, name)
-        return out
+                if isinstance(val, FrameAttribute) and name not in seen:
+                    seen.add(name)
+                    # Add the sort order, name, and actual value of the frame
+                    # attribute in question
+                    attributes.append((val._order, name,
+                                       getattr(mro_cls, name)))
+
+        # Sort by the frame attribute order
+        attributes.sort(key=lambda a: a[0])
+        return OrderedDict((a[1], a[2]) for a in attributes)
 
     @property
     def representation(self):
@@ -606,7 +626,12 @@ class BaseCoordinateFrame(object):
 
     def represent_as(self, new_representation, in_frame_units=False):
         """
-        Generate and return a new representation of this frame's `data`.
+        Generate and return a new representation of this frame's `data`
+        as a Representation object.
+
+        Note: In order to make an in-place change of the representation
+        of a Frame or SkyCoord object, set the ``representation``
+        attribute of that object to the desired new representation.
 
         Parameters
         ----------
@@ -621,7 +646,7 @@ class BaseCoordinateFrame(object):
 
         Returns
         -------
-        newrep : whatever ``new_representation`` is
+        newrep : BaseRepresentation-derived object
             A new representation object of this frame's `data`.
 
         Raises
@@ -632,10 +657,14 @@ class BaseCoordinateFrame(object):
         Examples
         --------
         >>> from astropy import units as u
-        >>> from astropy.coordinates import ICRS, CartesianRepresentation
-        >>> coord = ICRS(0*u.deg, 0*u.deg)
+        >>> from astropy.coordinates import SkyCoord, CartesianRepresentation
+        >>> coord = SkyCoord(0*u.deg, 0*u.deg)
         >>> coord.represent_as(CartesianRepresentation)
-        <CartesianRepresentation x=1.0 , y=0.0 , z=0.0 >
+        <CartesianRepresentation x=1.0, y=0.0, z=0.0>
+
+        >>> coord.representation = CartesianRepresentation
+        >>> coord
+        <SkyCoord (ICRS): x=1.0, y=0.0, z=0.0>
         """
         new_representation = _get_repr_cls(new_representation)
 
