@@ -20,6 +20,7 @@ import os
 import subprocess
 import shutil
 import tempfile
+import types
 import warnings
 
 try:
@@ -33,6 +34,8 @@ except ImportError:
 from distutils.core import Command
 
 from .. import test
+from ..utils.exceptions import AstropyWarning
+from ..config import configuration
 
 if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
     import pytest
@@ -40,14 +43,14 @@ if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
 else:
     from ..extern import pytest as extern_pytest
 
-    if sys.version_info >= (3, 0):
+    if six.PY3:
         exec("def do_exec_def(co, loc): exec(co, loc)\n")
         extern_pytest.do_exec = do_exec_def
 
         unpacked_sources = extern_pytest.sources.encode("ascii")
         unpacked_sources = pickle.loads(
-            zlib.decompress(base64.decodebytes(unpacked_sources)))
-    else:
+            zlib.decompress(base64.decodebytes(unpacked_sources)), encoding='utf-8')
+    elif six.PY2:
         exec("def do_exec_def(co, loc): exec co in loc\n")
         extern_pytest.do_exec = do_exec_def
 
@@ -61,7 +64,7 @@ else:
     # argparse before importing py.test, since it isn't in the
     # standard library, and py.test's workaround doesn't appear to
     # work with the "absolute imports" of Python 3.x.
-    if sys.version_info[0] == 3 and sys.version_info[1] <= 1:
+    if six.PY3 and sys.version_info[1] <= 1:
         argparse = importer.load_module(str('argparse'))
     pytest = importer.load_module(str('pytest'))
 
@@ -107,16 +110,26 @@ class TestRunner(object):
     def run_tests(self, package=None, test_path=None, args=None, plugins=None,
                   verbose=False, pastebin=None, remote_data=False, pep8=False,
                   pdb=False, coverage=False, open_files=False, parallel=0,
-                  docs_path=None, skip_docs=False):
+                  docs_path=None, skip_docs=False, repeat=None):
         """
         The docstring for this method lives in astropy/__init__.py:test
         """
-        # This prevents cyclical import problems that make it
-        # impossible to test packages that define Table types on their
-        # own.
-        from ..table import Table
+        try:
+            get_ipython()
+        except NameError:
+            pass
+        else:
+            raise RuntimeError(
+                "Running astropy tests inside of IPython is not supported.")
 
-        all_args = ''
+        if coverage:
+            warnings.warn(
+                "The coverage option is ignored on run_tests, since it "
+                "can not be made to work in that context.  Use "
+                "'python setup.py test --coverage' instead.",
+                AstropyWarning)
+
+        all_args = []
 
         if package is None:
             package_path = self.base_path
@@ -141,7 +154,7 @@ class TestRunner(object):
             base, ext = os.path.splitext(test_path)
             if ext == '.py':
                 test_path = os.path.abspath(test_path)
-                all_args += test_path
+                all_args.append(test_path)
             elif ext == '.rst':
                 if docs_path is None:
                     # This shouldn't happen from "python setup.py test"
@@ -152,34 +165,35 @@ class TestRunner(object):
                     # the astropy tree, we need to forcibly load the
                     # astropy py.test plugins, and then turn on the
                     # doctest_rst plugin.
-                    all_args += ' -p astropy.tests.pytest_plugins --doctest-rst '
+                    all_args.extend(['-p', 'astropy.tests.pytest_plugins', '--doctest-rst'])
                     test_path = os.path.join(docs_path, '..', test_path)
-                    all_args += test_path
+                    all_args.append(test_path)
             else:
                 raise ValueError("Test file path must be to a .py or .rst file")
         else:
-            all_args = package_path
+            all_args.append(package_path)
             if docs_path is not None and not skip_docs:
-                all_args += ' ' + docs_path + ' --doctest-rst '
+                all_args.extend([docs_path, '--doctest-rst'])
 
         # add any additional args entered by the user
         if args is not None:
-            all_args += ' {0}'.format(args)
+            all_args.extend(
+                shlex.split(args, posix=not sys.platform.startswith('win')))
 
         # add verbosity flag
         if verbose:
-            all_args += ' -v'
+            all_args.append('-v')
 
         # turn on pastebin output
         if pastebin is not None:
             if pastebin in ['failed', 'all']:
-                all_args += ' --pastebin={0}'.format(pastebin)
+                all_args.append('--pastebin={0}'.format(pastebin))
             else:
                 raise ValueError("pastebin should be 'failed' or 'all'")
 
         # run @remote_data tests
         if remote_data:
-            all_args += ' --remote-data'
+            all_args.append('--remote-data')
 
         if pep8:
             try:
@@ -188,43 +202,11 @@ class TestRunner(object):
                 raise ImportError('PEP8 checking requires pytest-pep8 plugin: '
                                   'http://pypi.python.org/pypi/pytest-pep8')
             else:
-                all_args += ' --pep8 -k pep8'
+                all_args.extend(['--pep8', '-k', 'pep8'])
 
         # activate post-mortem PDB for failing tests
         if pdb:
-            all_args += ' --pdb'
-
-        if coverage:
-            try:
-                import pytest_cov
-            except ImportError:
-                raise ImportError(
-                    'Coverage reporting requires pytest-cov plugin: '
-                    'http://pypi.python.org/pypi/pytest-cov')
-            else:
-                # Don't use get_pkg_data_filename here, because it
-                # requires importing astropy.config and thus screwing
-                # up coverage results for those packages.
-                coveragerc = os.path.join(
-                    package_path, 'tests', 'coveragerc')
-
-                # We create a coveragerc that is specific to the version
-                # of Python we're running, so that we can mark branches
-                # as being specifically for Python 2 or Python 3
-                with open(coveragerc, 'r') as fd:
-                    coveragerc_content = fd.read()
-                if sys.version_info[0] >= 3:
-                    ignore_python_version = '2'
-                else:
-                    ignore_python_version = '3'
-                coveragerc_content = coveragerc_content.replace(
-                    "{ignore_python_version}", ignore_python_version)
-                with tempfile.NamedTemporaryFile(delete=False) as tmp:
-                    tmp.write(coveragerc_content.encode('utf-8'))
-
-                all_args += (
-                    ' --cov-report html --cov astropy'
-                    ' --cov-config {0}'.format(tmp.name))
+            all_args.append('--pdb')
 
         # check for opened files after each test
         if open_files:
@@ -238,7 +220,7 @@ class TestRunner(object):
                     "open file detection requested, but could not "
                     "successfully run the 'lsof' command")
 
-            all_args += ' --open-files'
+            all_args.append('--open-files')
 
             print("Checking for unclosed files")
 
@@ -258,21 +240,48 @@ class TestRunner(object):
 
             if parallel < 0:
                 parallel = multiprocessing.cpu_count()
-            all_args += ' -n {0}'.format(parallel)
+            all_args.extend(['-n', six.text_type(parallel)])
 
-        if sys.version_info < (2, 7, 3):
-            all_args = all_args.encode('utf-8')
+        if repeat:
+            all_args.append('--repeat={0}'.format(repeat))
+
+        if six.PY2:
+            all_args = [x.encode('utf-8') for x in all_args]
+
+        # override the config locations to not make a new directory nor use
+        # existing cache or config
+        xdg_config_home = os.environ.get('XDG_CONFIG_HOME')
+        xdg_cache_home = os.environ.get('XDG_CACHE_HOME')
+        astropy_config = tempfile.mkdtemp('astropy_config')
+        astropy_cache = tempfile.mkdtemp('astropy_cache')
+        os.environ[str('XDG_CONFIG_HOME')] = str(astropy_config)
+        os.environ[str('XDG_CACHE_HOME')] = str(astropy_cache)
+        os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
+        os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
+        # To fully force configuration reloading from a different file (in this
+        # case our default one in a temp directory), clear the config object
+        # cache.
+        configuration._cfgobjs.clear()
+
+        # This prevents cyclical import problems that make it
+        # impossible to test packages that define Table types on their
+        # own.
+        from ..table import Table
 
         try:
-            all_args = shlex.split(
-                all_args, posix=not sys.platform.startswith('win'))
-
             result = pytest.main(args=all_args, plugins=plugins)
         finally:
-            if coverage:
-                if not tmp.closed:
-                    tmp.close()
-                os.remove(tmp.name)
+            shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
+            shutil.rmtree(os.environ['XDG_CACHE_HOME'])
+            if xdg_config_home is not None:
+                os.environ[str('XDG_CONFIG_HOME')] = xdg_config_home
+            else:
+                del os.environ['XDG_CONFIG_HOME']
+            if xdg_cache_home is not None:
+                os.environ[str('XDG_CACHE_HOME')] = xdg_cache_home
+            else:
+                del os.environ['XDG_CACHE_HOME']
+            configuration._cfgobjs.clear()
 
         return result
 
@@ -291,6 +300,254 @@ def _fix_user_options(options):
     return [tuple(to_str_or_none(x) for x in y) for y in options]
 
 
+def _save_coverage(cov, result, rootdir, testing_path):
+    """
+    This method is called after the tests have been run in coverage mode
+    to cleanup and then save the coverage data and report.
+    """
+    from ..utils.console import color_print
+
+    if result != 0:
+        return
+
+    # The coverage report includes the full path to the temporary
+    # directory, so we replace all the paths with the true source
+    # path. This means that the coverage line-by-line report will only
+    # be correct for Python 2 code (since the Python 3 code will be
+    # different in the build directory from the source directory as
+    # long as 2to3 is needed). Therefore we only do this fix for
+    # Python 2.x.
+    if six.PY2:
+        d = cov.data
+        cov._harvest_data()
+        for key in d.lines.keys():
+            new_path = os.path.relpath(
+                os.path.realpath(key),
+                os.path.realpath(testing_path))
+            new_path = os.path.abspath(
+                os.path.join(rootdir, new_path))
+            d.lines[new_path] = d.lines.pop(key)
+
+    color_print('Saving coverage data in .coverage...', 'green')
+    cov.save()
+
+    color_print('Saving HTML coverage report in htmlcov...', 'green')
+    cov.html_report(directory=os.path.join(rootdir, 'htmlcov'))
+
+
+class raises(object):
+    """
+    A decorator to mark that a test should raise a given exception.
+    Use as follows::
+
+        @raises(ZeroDivisionError)
+        def test_foo():
+            x = 1/0
+
+    This can also be used a context manager, in which case it is just an alias
+    for the `pytest.raises` context manager (because the two have the same name
+    this help avoid confusion by being flexible).
+    """
+
+    # pep-8 naming exception -- this is a decorator class
+    def __init__(self, exc):
+        self._exc = exc
+        self._ctx = None
+
+    def __call__(self, func):
+        @functools.wraps(func)
+        def run_raises_test(*args, **kwargs):
+            pytest.raises(self._exc, func, *args, **kwargs)
+        return run_raises_test
+
+    def __enter__(self):
+        self._ctx = pytest.raises(self._exc)
+        return self._ctx.__enter__()
+
+    def __exit__(self, *exc_info):
+        return self._ctx.__exit__(*exc_info)
+
+
+_deprecations_as_exceptions = False
+
+
+def enable_deprecations_as_exceptions():
+    """
+    Turn on the feature that turns deprecations into exceptions.
+    """
+    global _deprecations_as_exceptions
+    _deprecations_as_exceptions = True
+
+
+def treat_deprecations_as_exceptions():
+    """
+    Turn all DeprecationWarnings (which indicate deprecated uses of
+    Python itself or Numpy, but not within Astropy, where we use our
+    own deprecation warning class) into exceptions so that we find
+    out about them early.
+
+    This completely resets the warning filters and any "already seen"
+    warning state.
+    """
+    if not _deprecations_as_exceptions:
+        return
+
+    # First, totally reset the warning state
+    for module in list(six.itervalues(sys.modules)):
+        # We don't want to deal with six.MovedModules, only "real"
+        # modules.
+        if (isinstance(module, types.ModuleType) and
+            hasattr(module, '__warningregistry__')):
+            del module.__warningregistry__
+
+    warnings.resetwarnings()
+
+    # Hide the next couple of DeprecationWarnings
+    warnings.simplefilter('ignore', DeprecationWarning)
+    # Here's the wrinkle: a couple of our third-party dependencies
+    # (py.test and scipy) are still using deprecated features
+    # themselves, and we'd like to ignore those.  Fortunately, those
+    # show up only at import time, so if we import those things *now*,
+    # before we turn the warnings into exceptions, we're golden.
+    try:
+        # A deprecated stdlib module used by py.test
+        import compiler
+    except ImportError:
+        pass
+
+    try:
+        import scipy
+    except ImportError:
+        pass
+
+    # Now, start over again with the warning filters
+    warnings.resetwarnings()
+    # Now, turn DeprecationWarnings into exceptions
+    warnings.filterwarnings("error", ".*", DeprecationWarning)
+
+    if sys.version_info[:2] == (2, 6):
+        # py.test's warning.showwarning does not include the line argument
+        # on Python 2.6, so we need to explicitly ignore this warning.
+        warnings.filterwarnings(
+            "always",
+            r"functions overriding warnings\.showwarning\(\) must support "
+            r"the 'line' argument",
+            DeprecationWarning)
+
+    if sys.version_info[:2] >= (3, 4):
+        # py.test reads files with the 'U' flag, which is now
+        # deprecated in Python 3.4.
+        warnings.filterwarnings(
+            "always",
+            r"'U' mode is deprecated",
+            DeprecationWarning)
+
+
+class catch_warnings(warnings.catch_warnings):
+    """
+    A high-powered version of warnings.catch_warnings to use for testing
+    and to make sure that there is no dependence on the order in which
+    the tests are run.
+
+    This completely blitzes any memory of any warnings that have
+    appeared before so that all warnings will be caught and displayed.
+
+    *args is a set of warning classes to collect.  If no arguments are
+    provided, all warnings are collected.
+
+    Use as follows::
+
+        with catch_warnings(MyCustomWarning) as w:
+            do.something.bad()
+        assert len(w) > 0
+    """
+    def __init__(self, *classes):
+        super(catch_warnings, self).__init__(record=True)
+        self.classes = classes
+
+    def __enter__(self):
+        warning_list = super(catch_warnings, self).__enter__()
+        treat_deprecations_as_exceptions()
+        if len(self.classes) == 0:
+            warnings.simplefilter('always')
+        else:
+            warnings.simplefilter('ignore')
+            for cls in self.classes:
+                warnings.simplefilter('always', cls)
+        return warning_list
+
+    def __exit__(self, type, value, traceback):
+        treat_deprecations_as_exceptions()
+
+
+def assert_follows_unicode_guidelines(
+        x, roundtrip=None):
+    """
+    Test that an object follows our Unicode policy.  See
+    "Unicode Policy" in the coding guidelines.
+
+    Parameters
+    ----------
+    x : object
+        The instance to test
+
+    roundtrip : module, optional
+        When provided, this namespace will be used to evaluate
+        ``repr(x)`` and ensure that it roundtrips.  It will also
+        ensure that ``__bytes__(x)`` and ``__unicode__(x)`` roundtrip.
+        If not provided, no roundtrip testing will be performed.
+    """
+    from .. import conf
+    from ..extern import six
+
+    with conf.set_temp('unicode_output', False):
+        bytes_x = bytes(x)
+        unicode_x = six.text_type(x)
+        repr_x = repr(x)
+
+        assert isinstance(bytes_x, bytes)
+        bytes_x.decode('ascii')
+        assert isinstance(unicode_x, six.text_type)
+        unicode_x.encode('ascii')
+        assert isinstance(repr_x, six.string_types)
+        if isinstance(repr_x, bytes):
+            repr_x.decode('ascii')
+        else:
+            repr_x.encode('ascii')
+
+        if roundtrip is not None:
+            assert x.__class__(bytes_x) == x
+            assert x.__class__(unicode_x) == x
+            assert eval(repr_x, roundtrip) == x
+
+    with conf.set_temp('unicode_output', True):
+        bytes_x = bytes(x)
+        unicode_x = six.text_type(x)
+        repr_x = repr(x)
+
+        assert isinstance(bytes_x, bytes)
+        bytes_x.decode('ascii')
+        assert isinstance(unicode_x, six.text_type)
+        assert isinstance(repr_x, six.string_types)
+        if isinstance(repr_x, bytes):
+            repr_x.decode('ascii')
+        else:
+            repr_x.encode('ascii')
+
+        if roundtrip is not None:
+            assert x.__class__(bytes_x) == x
+            assert x.__class__(unicode_x) == x
+            assert eval(repr_x, roundtrip) == x
+
+
+##############################################################################
+# Note: the following class exists only for backward-compatibility purposes. #
+#       It has been moved to the separate astropy-helpers package, located   #
+#       at https://github.com/astropy/astropy-helpers. Any new development   #
+#       or bug fixes should be done there                                    #
+##############################################################################
+
+
 class astropy_test(Command, object):
     user_options = [
         ('package=', 'P',
@@ -316,10 +573,9 @@ class astropy_test(Command, object):
         ('pdb', 'd',
          'Start the interactive Python debugger on errors.'),
         ('coverage', 'c',
-         'Create a coverage report. Requires the pytest-cov '
-         'plugin.'),
+         'Create a coverage report. Requires the coverage package.'),
         ('open-files', 'o', 'Fail if any tests leave files open.'),
-        ('parallel=', 'n',
+        ('parallel=', 'j',
          'Run the tests in parallel on the specified number of '
          'CPUs.  If negative, all the cores on the machine will be '
          'used.  Requires the pytest-xdist plugin.'),
@@ -362,6 +618,10 @@ class astropy_test(Command, object):
         build_cmd = self.get_finalized_command('build')
         new_path = os.path.abspath(build_cmd.build_lib)
 
+        if self.docs_path is None:
+            if os.path.exists('docs'):
+                self.docs_path = os.path.abspath('docs')
+
         # Copy the build to a temporary directory for the purposes of testing
         # - this avoids creating pyc and __pycache__ directories inside the
         # build directory
@@ -370,25 +630,61 @@ class astropy_test(Command, object):
         shutil.copytree(new_path, testing_path)
         shutil.copy('setup.cfg', testing_path)
 
-        if self.docs_path is None:
-            if os.path.exists('docs'):
-                self.docs_path = os.path.abspath('docs')
+        cmd_pre = ''
+        cmd_post = ''
 
         try:
+            if self.coverage:
+                if self.parallel != 0:
+                    raise ValueError(
+                        "--coverage can not be used with --parallel")
 
-            # Run the tests in a subprocess--this is necessary since new extension
-            # modules may have appeared, and this is the easiest way to set up a
-            # new environment
+                try:
+                    import coverage
+                except ImportError:
+                    raise ImportError(
+                        "--coverage requires that the coverage package is "
+                        "installed.")
 
-            # We need to set a flag in the child's environment so that
-            # unnecessary code is not imported before py.test can start
-            # up, otherwise the coverage results will be artifically low.
-            if sys.version_info[0] >= 3:
+                # Don't use get_pkg_data_filename here, because it
+                # requires importing astropy.config and thus screwing
+                # up coverage results for those packages.
+                coveragerc = os.path.join(
+                    testing_path, self.package_name, 'tests', 'coveragerc')
+
+                # We create a coveragerc that is specific to the version
+                # of Python we're running, so that we can mark branches
+                # as being specifically for Python 2 or Python 3
+                with open(coveragerc, 'rb') as fd:
+                    coveragerc_content = fd.read().decode('utf-8')
+                if six.PY3:
+                    ignore_python_version = '2'
+                elif six.PY2:
+                    ignore_python_version = '3'
+                coveragerc_content = coveragerc_content.replace(
+                    "{ignore_python_version}", ignore_python_version).replace(
+                        "{packagename}", self.package_name)
+                tmp_coveragerc = os.path.join(tmp_dir, 'coveragerc')
+                with open(tmp_coveragerc, 'wb') as tmp:
+                    tmp.write(coveragerc_content.encode('utf-8'))
+
+                cmd_pre = (
+                    'import coverage; '
+                    'cov = coverage.coverage(data_file="{0}", config_file="{1}"); '
+                    'cov.start();'.format(
+                        os.path.abspath(".coverage"), tmp_coveragerc))
+                cmd_post = (
+                    'cov.stop(); '
+                    'from astropy.tests.helper import _save_coverage; '
+                    '_save_coverage(cov, result, "{0}", "{1}");'.format(
+                        os.path.abspath('.'), testing_path))
+
+            if six.PY3:
                 set_flag = "import builtins; builtins._ASTROPY_TEST_ = True"
-            else:
+            elif six.PY2:
                 set_flag = "import __builtin__; __builtin__._ASTROPY_TEST_ = True"
 
-            cmd = ('{0}; import {1.package_name}, sys; sys.exit('
+            cmd = ('{cmd_pre}{0}; import {1.package_name}, sys; result = ('
                    '{1.package_name}.test('
                    'package={1.package!r}, '
                    'test_path={1.test_path!r}, '
@@ -399,192 +695,35 @@ class astropy_test(Command, object):
                    'remote_data={1.remote_data!r}, '
                    'pep8={1.pep8!r}, '
                    'pdb={1.pdb!r}, '
-                   'coverage={1.coverage!r}, '
                    'open_files={1.open_files!r}, '
                    'parallel={1.parallel!r}, '
                    'docs_path={1.docs_path!r}, '
-                   'skip_docs={1.skip_docs!r}))')
-            cmd = cmd.format(set_flag, self)
+                   'skip_docs={1.skip_docs!r})); '
+                   '{cmd_post}'
+                   'sys.exit(result)')
+            cmd = cmd.format(set_flag, self, cmd_pre=cmd_pre, cmd_post=cmd_post)
 
-            # override the config locations to not make a new directory nor use
-            # existing cache or config
-            os.environ['XDG_CONFIG_HOME'] = tempfile.mkdtemp('astropy_config')
-            os.environ['XDG_CACHE_HOME'] = tempfile.mkdtemp('astropy_cache')
-            os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
-            os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
-
-            try:
-                retcode = subprocess.call([sys.executable, '-B', '-c', cmd],
-                                          cwd=testing_path, close_fds=False)
-            finally:
-                # kill the temporary dirs
-                shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
-                shutil.rmtree(os.environ['XDG_CACHE_HOME'])
-
-            if self.coverage and retcode == 0:
-
-                # Copy the htmlcov from build/lib.../htmlcov to a more
-                # obvious place
-                if os.path.exists('htmlcov'):
-                    shutil.rmtree('htmlcov')
-                shutil.copytree(os.path.join(testing_path, 'htmlcov'), 'htmlcov')
-
-                # The coverage report includes the full path to the temporary
-                # directory, so we replace all the paths with the true source
-                # path. This means that the coverage line-by-line report will
-                # only be correct for Python 2 code (since the Python 3 code
-                # will be different in the build directory from the source
-                # directory as long as 2to3 is needed). Therefore we only do
-                # this fix for Python 2.x.
-
-                if six.PY2:
-                    import coverage
-                    d = coverage.CoverageData()
-                    d.read_file(os.path.join(testing_path, '.coverage'))
-                    for key in d.lines.keys():
-                        new_path = os.path.relpath(os.path.realpath(key),
-                                                   os.path.realpath(testing_path))
-                        new_path = os.path.abspath(new_path)
-                        d.lines[new_path] = d.lines.pop(key)
-                    d.write()
-                else:
-                    shutil.copy2(os.path.join(testing_path, '.coverage'), '.coverage')
+            # Run the tests in a subprocess--this is necessary since
+            # new extension modules may have appeared, and this is the
+            # easiest way to set up a new environment
 
+            # On Python 3.x prior to 3.3, the creation of .pyc files
+            # is not atomic.  py.test jumps through some hoops to make
+            # this work by parsing import statements and carefully
+            # importing files atomically.  However, it can't detect
+            # when __import__ is used, so its carefulness still fails.
+            # The solution here (admittedly a bit of a hack), is to
+            # turn off the generation of .pyc files altogether by
+            # passing the `-B` switch to `python`.  This does mean
+            # that each core will have to compile .py file to bytecode
+            # itself, rather than getting lucky and borrowing the work
+            # already done by another core.  Compilation is an
+            # insignificant fraction of total testing time, though, so
+            # it's probably not worth worrying about.
+            retcode = subprocess.call([sys.executable, '-B', '-c', cmd],
+                                      cwd=testing_path, close_fds=False)
         finally:
-
             # Remove temporary directory
             shutil.rmtree(tmp_dir)
 
         raise SystemExit(retcode)
-
-
-class raises(object):
-    """
-    A decorator to mark that a test should raise a given exception.
-    Use as follows::
-
-        @raises(ZeroDivisionError)
-        def test_foo():
-            x = 1/0
-
-    This can also be used a context manager, in which case it is just an alias
-    for the `pytest.raises` context manager (because the two have the same name
-    this help avoid confusion by being flexible).
-    """
-
-    # pep-8 naming exception -- this is a decorator class
-    def __init__(self, exc):
-        self._exc = exc
-        self._ctx = None
-
-    def __call__(self, func):
-        @functools.wraps(func)
-        def run_raises_test(*args, **kwargs):
-            pytest.raises(self._exc, func, *args, **kwargs)
-        return run_raises_test
-
-    def __enter__(self):
-        self._ctx = pytest.raises(self._exc)
-        return self._ctx.__enter__()
-
-    def __exit__(self, *exc_info):
-        return self._ctx.__exit__(*exc_info)
-
-
-class catch_warnings(warnings.catch_warnings):
-    """
-    A high-powered version of warnings.catch_warnings to use for testing
-    and to make sure that there is no dependence on the order in which
-    the tests are run.
-
-    This completely blitzes any memory of any warnings that have
-    appeared before so that all warnings will be caught and displayed.
-
-    *args is a set of warning classes to collect.  If no arguments are
-    provided, all warnings are collected.
-
-    Use as follows::
-
-        with catch_warnings(MyCustomWarning) as w:
-            do.something.bad()
-        assert len(w) > 0
-    """
-    def __init__(self, *classes):
-        for module in list(six.itervalues(sys.modules)):
-            if hasattr(module, '__warningregistry__'):
-                del module.__warningregistry__
-        super(catch_warnings, self).__init__(record=True)
-        self.classes = classes
-
-    def __enter__(self):
-        warning_list = super(catch_warnings, self).__enter__()
-        warnings.resetwarnings()
-        if len(self.classes) == 0:
-            warnings.simplefilter('always')
-        else:
-            warnings.simplefilter('ignore')
-            for cls in self.classes:
-                warnings.simplefilter('always', cls)
-        return warning_list
-
-
-def assert_follows_unicode_guidelines(
-        x, roundtrip=None):
-    """
-    Test that an object follows our Unicode policy.  See
-    "Unicode Policy" in the coding guidelines.
-
-    Parameters
-    ----------
-    x : object
-        The instance to test
-
-    roundtrip : module, optional
-        When provided, this namespace will be used to evaluate
-        ``repr(x)`` and ensure that it roundtrips.  It will also
-        ensure that ``__bytes__(x)`` and ``__unicode__(x)`` roundtrip.
-        If not provided, no roundtrip testing will be performed.
-    """
-    from .. import UNICODE_OUTPUT
-    from ..extern import six
-
-    UNICODE_OUTPUT.set(False)
-
-    bytes_x = bytes(x)
-    unicode_x = six.text_type(x)
-    repr_x = repr(x)
-
-    assert isinstance(bytes_x, bytes)
-    bytes_x.decode('ascii')
-    assert isinstance(unicode_x, six.text_type)
-    unicode_x.encode('ascii')
-    assert isinstance(repr_x, six.string_types)
-    if isinstance(repr_x, bytes):
-        repr_x.decode('ascii')
-    else:
-        repr_x.encode('ascii')
-
-    if roundtrip is not None:
-        assert x.__class__(bytes_x) == x
-        assert x.__class__(unicode_x) == x
-        assert eval(repr_x, roundtrip) == x
-
-    UNICODE_OUTPUT.set(True)
-
-    bytes_x = bytes(x)
-    unicode_x = six.text_type(x)
-    repr_x = repr(x)
-
-    assert isinstance(bytes_x, bytes)
-    bytes_x.decode('ascii')
-    assert isinstance(unicode_x, six.text_type)
-    assert isinstance(repr_x, six.string_types)
-    if isinstance(repr_x, bytes):
-        repr_x.decode('ascii')
-    else:
-        repr_x.encode('ascii')
-
-    if roundtrip is not None:
-        assert x.__class__(bytes_x) == x
-        assert x.__class__(unicode_x) == x
-        assert eval(repr_x, roundtrip) == x
