@@ -1,20 +1,23 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-"""
-A set of standard astronomical equivalencies.
-"""
-
+"""A set of standard astronomical equivalencies."""
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+# THIRD-PARTY
+import numpy as np
+
+# LOCAL
 from ..constants import si as _si
 from . import si
 from . import cgs
 from . import astrophys
+from . import dimensionless_unscaled
+
 
 __all__ = ['parallax', 'spectral', 'spectral_density', 'doppler_radio',
            'doppler_optical', 'doppler_relativistic', 'mass_energy',
-           'brightness_temperature', 'dimensionless_angles']
+           'brightness_temperature', 'dimensionless_angles',
+           'logarithmic', 'temperature', 'temperature_energy']
 
 
 def dimensionless_angles():
@@ -22,11 +25,19 @@ def dimensionless_angles():
 
     It is special compared to other equivalency pairs in that it
     allows this independent of the power to which the angle is raised,
-    and indepedent of whether it is part of a more complicated unit.
+    and independent of whether it is part of a more complicated unit.
     """
     return [(si.radian, None)]
 
 
+def logarithmic():
+    """Allow logarithmic units to be converted to dimensionless fractions"""
+    return [
+        (dimensionless_unscaled, astrophys.dex,
+         lambda x: np.log10(x), lambda x: 10.**x)
+    ]
+
+
 def parallax():
     """
     Returns a list of equivalence pairs that handle the conversion
@@ -45,16 +56,30 @@ def spectral():
     Allows conversions between wavelength units, wave number units,
     frequency units, and energy units as they relate to light.
 
+    There are two types of wave number:
+
+        * spectroscopic - :math:`1 / \\lambda` (per meter)
+        * angular - :math:`2 \\pi / \\lambda` (radian per meter)
+
     """
     hc = _si.h.value * _si.c.value
-    inv_m = si.m ** -1
+    two_pi = 2.0 * np.pi
+    inv_m_spec = si.m ** -1
+    inv_m_ang = si.radian / si.m
+
     return [
         (si.m, si.Hz, lambda x: _si.c.value / x),
         (si.m, si.J, lambda x: hc / x),
-        (si.m, inv_m, lambda x: 1.0 / x),
         (si.Hz, si.J, lambda x: _si.h.value * x, lambda x: x / _si.h.value),
-        (si.Hz, inv_m, lambda x: x / _si.c.value, lambda x: _si.c.value * x),
-        (si.J, inv_m, lambda x: x / hc, lambda x: hc * x)
+        (si.m, inv_m_spec, lambda x: 1.0 / x),
+        (si.Hz, inv_m_spec, lambda x: x / _si.c.value,
+         lambda x: _si.c.value * x),
+        (si.J, inv_m_spec, lambda x: x / hc, lambda x: hc * x),
+        (inv_m_spec, inv_m_ang, lambda x: x * two_pi, lambda x: x / two_pi),
+        (si.m, inv_m_ang, lambda x: two_pi / x),
+        (si.Hz, inv_m_ang, lambda x: two_pi * x / _si.c.value,
+         lambda x: _si.c.value * x / two_pi),
+        (si.J, inv_m_ang, lambda x: x * two_pi / hc, lambda x: hc * x / two_pi)
     ]
 
 
@@ -65,8 +90,8 @@ def spectral_density(wav, factor=None):
 
     Parameters
     ----------
-    wav : Quantity
-        Quantity associated with values being converted
+    wav : `~astropy.units.Quantity`
+        `~astropy.units.Quantity` associated with values being converted
         (e.g., wavelength or frequency).
 
     Notes
@@ -81,7 +106,7 @@ def spectral_density(wav, factor=None):
     if isinstance(wav, UnitBase):
         if factor is None:
             raise ValueError(
-                'If ``wav`` is specified as a unit, ``factor`` should be set')
+                'If `wav` is specified as a unit, `factor` should be set')
         wav = factor * wav   # Convert to Quantity
 
     c_Aps = _si.c.to(si.AA / si.s).value  # Angstrom/s
@@ -142,10 +167,7 @@ def spectral_density(wav, factor=None):
         return x * wav.to(si.AA, spectral()).value ** 3 / (hc * c_Aps)
 
     return [
-        (si.AA, fnu, converter, iconverter),
         (fla, fnu, converter, iconverter),
-        (si.AA, si.Hz, converter, iconverter),
-        (fla, si.Hz, converter, iconverter),
         (fnu, nufnu, converter_fnu_nufnu, iconverter_fnu_nufnu),
         (fla, lafla, converter_fla_lafla, iconverter_fla_lafla),
         (photlam, fla, converter_photlam_fla, iconverter_photlam_fla),
@@ -166,7 +188,7 @@ def doppler_radio(rest):
 
     Parameters
     ----------
-    rest : Quantity
+    rest : `~astropy.units.Quantity`
         Any quantity supported by the standard spectral equivalencies
         (wavelength, energy, frequency, wave number).
 
@@ -181,8 +203,8 @@ def doppler_radio(rest):
     >>> radio_CO_equiv = u.doppler_radio(CO_restfreq)
     >>> measured_freq = 115.2832*u.GHz
     >>> radio_velocity = measured_freq.to(u.km/u.s, equivalencies=radio_CO_equiv)
-    >>> radio_velocity
-    <Quantity -31.209092... km / s>
+    >>> radio_velocity  # doctest: +FLOAT_CMP
+    <Quantity -31.209092088877583 km / s>
     """
 
     ckms = _si.c.to('km/s').value
@@ -231,7 +253,7 @@ def doppler_optical(rest):
 
     Parameters
     ----------
-    rest : Quantity
+    rest : `~astropy.units.Quantity`
         Any quantity supported by the standard spectral equivalencies
         (wavelength, energy, frequency, wave number).
 
@@ -246,8 +268,8 @@ def doppler_optical(rest):
     >>> optical_CO_equiv = u.doppler_optical(CO_restfreq)
     >>> measured_freq = 115.2832*u.GHz
     >>> optical_velocity = measured_freq.to(u.km/u.s, equivalencies=optical_CO_equiv)
-    >>> optical_velocity
-    <Quantity -31.205843... km / s>
+    >>> optical_velocity  # doctest: +FLOAT_CMP
+    <Quantity -31.20584348799674 km / s>
     """
 
     ckms = _si.c.to('km/s').value
@@ -297,7 +319,7 @@ def doppler_relativistic(rest):
 
     Parameters
     ----------
-    rest : Quantity
+    rest : `~astropy.units.Quantity`
         Any quantity supported by the standard spectral equivalencies
         (wavelength, energy, frequency, wave number).
 
@@ -312,15 +334,15 @@ def doppler_relativistic(rest):
     >>> relativistic_CO_equiv = u.doppler_relativistic(CO_restfreq)
     >>> measured_freq = 115.2832*u.GHz
     >>> relativistic_velocity = measured_freq.to(u.km/u.s, equivalencies=relativistic_CO_equiv)
-    >>> relativistic_velocity
-    <Quantity -31.207467619...
+    >>> relativistic_velocity  # doctest: +FLOAT_CMP
+    <Quantity -31.207467619351537 km / s>
     >>> measured_velocity = 1250 * u.km/u.s
     >>> relativistic_frequency = measured_velocity.to(u.GHz, equivalencies=relativistic_CO_equiv)
-    >>> relativistic_frequency
-    <Quantity 114.7915686...
+    >>> relativistic_frequency  # doctest: +FLOAT_CMP
+    <Quantity 114.79156866993588 GHz>
     >>> relativistic_wavelength = measured_velocity.to(u.mm, equivalencies=relativistic_CO_equiv)
-    >>> relativistic_wavelength
-    <Quantity 2.6116243681...
+    >>> relativistic_wavelength  # doctest: +FLOAT_CMP
+    <Quantity 2.6116243681798923 mm>
     """
 
     ckms = _si.c.to('km/s').value
@@ -396,9 +418,9 @@ def brightness_temperature(beam_area, disp):
     ----------
     beam_area : Beam Area equivalent
         Beam area in angular units, i.e. steradian equivalent
-    disp : `Quantity` with spectral units
-        The observed `spectral` equivalent `Unit` (e.g., frequency or
-        wavelength)
+    disp : `~astropy.units.Quantity` with spectral units
+        The observed `spectral` equivalent `~astropy.units.Unit` (e.g.,
+        frequency or wavelength)
 
     Examples
     --------
@@ -408,17 +430,19 @@ def brightness_temperature(beam_area, disp):
         >>> from astropy import units as u
         >>> beam_area = np.pi*(50*u.arcsec)**2
         >>> freq = 5*u.GHz
-        >>> u.Jy.to(u.K, equivalencies=u.brightness_temperature(beam_area,freq))
-        7.052588858...
-        >>> (1*u.Jy).to(u.K, equivalencies=u.brightness_temperature(beam_area,freq))
-        <Quantity 7.05258...
+        >>> equiv = u.brightness_temperature(beam_area, freq)
+        >>> u.Jy.to(u.K, equivalencies=equiv)  # doctest: +FLOAT_CMP
+        7.052588858846446
+        >>> (1*u.Jy).to(u.K, equivalencies=equiv)  # doctest: +FLOAT_CMP
+        <Quantity 7.052588858846446 K>
 
     VLA synthetic beam::
 
         >>> beam_area = np.pi*(15*u.arcsec)**2
         >>> freq = 5*u.GHz
-        >>> u.Jy.to(u.K, equivalencies=u.brightness_temperature(beam_area,freq))
-        78.36209843...
+        >>> equiv = u.brightness_temperature(beam_area, freq)
+        >>> u.Jy.to(u.K, equivalencies=equiv)  # doctest: +FLOAT_CMP
+        78.36209843162719
     """
     beam = beam_area.to(si.sr).value
     nu = disp.to(si.GHz, spectral())
@@ -432,3 +456,21 @@ def brightness_temperature(beam_area, disp):
         return (x_K * beam / factor)
 
     return [(astrophys.Jy, si.K, convert_Jy_to_K, convert_K_to_Jy)]
+
+
+def temperature():
+    """Convert between Kelvin, Celsius, and Fahrenheit here because
+    Unit and CompositeUnit cannot do addition or subtraction properly.
+    """
+    from .imperial import deg_F
+    return [
+        (si.K, si.deg_C, lambda x: x - 273.15, lambda x: x + 273.15),
+        (si.deg_C, deg_F, lambda x: x * 1.8 + 32.0, lambda x: (x - 32.0) / 1.8),
+        (si.K, deg_F, lambda x: (x - 273.15) * 1.8 + 32.0,
+         lambda x: ((x - 32.0) / 1.8) + 273.15)]
+
+def temperature_energy():
+    """Convert between Kelvin and keV(eV) to an equivalent amount."""
+    return [
+        (si.K, si.eV, lambda x: x / (_si.e.value / _si.k_B),
+         lambda x: x * (_si.e.value / _si.k_B))]
