@@ -2,12 +2,12 @@
 # array methods returns quantities with the right units, or raises exceptions.
 
 import numpy as np
-from numpy.testing.utils import assert_allclose
 
 from ... import units as u
 from ...tests.helper import pytest
 
 NUMPY_LT_1P7 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 7]
+NUMPY_LT_1P8 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 8]
 
 
 class TestQuantityArrayCopy(object):
@@ -76,6 +76,54 @@ class TestQuantityArrayCopy(object):
         assert q[2,2] == -1. * u.km / u.s
 
 
+class TestQuantityReshapeFuncs(object):
+    """Test different ndarray methods that alter the array shape
+
+    tests: reshape, squeeze, ravel, flatten, transpose, swapaxes
+    """
+    def test_reshape(self):
+        q = np.arange(6.) * u.m
+        q_reshape = q.reshape(3, 2)
+        assert isinstance(q_reshape, u.Quantity)
+        assert q_reshape.unit == q.unit
+        assert np.all(q_reshape.value == q.value.reshape(3, 2))
+
+    def test_squeeze(self):
+        q = np.arange(6.).reshape(6, 1) * u.m
+        q_squeeze = q.squeeze()
+        assert isinstance(q_squeeze, u.Quantity)
+        assert q_squeeze.unit == q.unit
+        assert np.all(q_squeeze.value == q.value.squeeze())
+
+    def test_ravel(self):
+        q = np.arange(6.).reshape(3, 2) * u.m
+        q_ravel = q.ravel()
+        assert isinstance(q_ravel, u.Quantity)
+        assert q_ravel.unit == q.unit
+        assert np.all(q_ravel.value == q.value.ravel())
+
+    def test_flatten(self):
+        q = np.arange(6.).reshape(3, 2) * u.m
+        q_flatten = q.flatten()
+        assert isinstance(q_flatten, u.Quantity)
+        assert q_flatten.unit == q.unit
+        assert np.all(q_flatten.value == q.value.flatten())
+
+    def test_transpose(self):
+        q = np.arange(6.).reshape(3, 2) * u.m
+        q_transpose = q.transpose()
+        assert isinstance(q_transpose, u.Quantity)
+        assert q_transpose.unit == q.unit
+        assert np.all(q_transpose.value == q.value.transpose())
+
+    def test_swapaxes(self):
+        q = np.arange(6.).reshape(3, 1, 2) * u.m
+        q_swapaxes = q.swapaxes(0, 2)
+        assert isinstance(q_swapaxes, u.Quantity)
+        assert q_swapaxes.unit == q.unit
+        assert np.all(q_swapaxes.value == q.value.swapaxes(0, 2))
+
+
 class TestQuantityStatsFuncs(object):
     """
     Test statistical functions
@@ -88,7 +136,8 @@ class TestQuantityStatsFuncs(object):
     def test_mean_inplace(self):
         q1 = np.array([1., 2., 4., 5., 6.]) * u.m
         qi = 1.5 * u.s
-        np.mean(q1, out=qi)
+        qi2 = np.mean(q1, out=qi)
+        assert qi2 is qi
         assert qi == 3.6 * u.m
 
     def test_std(self):
@@ -97,8 +146,11 @@ class TestQuantityStatsFuncs(object):
 
     def test_std_inplace(self):
 
-        # can't use decorator since test causes a segfault in Numpy < 1.7, and
-        # py.test will run the test anyway to see if it works
+        # For Numpy >= 1.7, inplace causes the variance to be stored instead
+        # of the standard deviation.
+        # see https://github.com/numpy/numpy/issues/5240
+        # For Numpy < 1.7, the test segfaults.  Hence, we cannot use the xfail
+        # decorator since py.test will run the test anyway to see if it works.
         pytest.xfail()
 
         q1 = np.array([1., 2.]) * u.m
@@ -112,8 +164,8 @@ class TestQuantityStatsFuncs(object):
 
     def test_var_inplace(self):
 
-        # can't use decorator since test causes a segfault in Numpy < 1.7, and
-        # py.test will run the test anyway to see if it works
+        # For Numpy < 1.7, the test segfaults.  Hence, we cannot use the xfail
+        # decorator since py.test will run the test anyway to see if it works.
         if NUMPY_LT_1P7:
             pytest.xfail()
 
@@ -188,8 +240,19 @@ class TestQuantityStatsFuncs(object):
         assert qi == 5. * u.m
 
     def test_round(self):
-        q1 = np.array([1.2, 2.2, 3.2]) * u.kg
+        q1 = np.array([1.253, 2.253, 3.253]) * u.kg
         assert np.all(np.round(q1) == np.array([1, 2, 3]) * u.kg)
+        assert np.all(np.round(q1, decimals=2) ==
+                      np.round(q1.value, decimals=2) * u.kg)
+        assert np.all(q1.round(decimals=2) ==
+                      q1.value.round(decimals=2) * u.kg)
+
+    def test_round_inplace(self):
+        q1 = np.array([1.253, 2.253, 3.253]) * u.kg
+        qi = np.zeros(3) * u.s
+        a = q1.round(decimals=2, out=qi)
+        assert a is qi
+        assert np.all(q1.round(decimals=2) == qi)
 
     def test_sum(self):
 
@@ -236,6 +299,20 @@ class TestQuantityStatsFuncs(object):
         assert np.all(q2.nansum(0) == np.array([1., 5., 10.]) * u.s)
         assert np.all(np.nansum(q2, 0) == np.array([1., 5., 10.]) * u.s)
 
+    @pytest.mark.xfail("NUMPY_LT_1P8")
+    def test_nansum_inplace(self):
+
+        q1 = np.array([1., 2., np.nan]) * u.m
+        qi = 1.5 * u.s
+        qout = q1.nansum(out=qi)
+        assert qout is qi
+        assert qi == np.nansum(q1.value) * q1.unit
+
+        qi2 = 1.5 * u.s
+        qout2 = np.nansum(q1, out=qi2)
+        assert qout2 is qi2
+        assert qi2 == np.nansum(q1.value) * q1.unit
+
     def test_prod(self):
 
         q1 = np.array([1, 2, 6]) * u.m
@@ -338,7 +415,7 @@ class TestArrayConversion(object):
     """
 
     def test_item(self):
-        q1 = np.array([1, 2, 3]) * u.m / u.km
+        q1 = u.Quantity(np.array([1, 2, 3]), u.m / u.km, dtype=int)
         assert q1.item(1) == 2 * q1.unit
         q1.itemset(1, 1)
         assert q1.item(1) == 1000 * u.m / u.km
@@ -364,19 +441,30 @@ class TestArrayConversion(object):
         assert q1.item(0) == 5 * u.m / u.km
 
     def test_slice(self):
-        q2 = np.array([[1, 2, 3], [4, 5, 6]]) * u.km / u.m
+        """Test that setitem changes the unit if needed (or ignores it for
+        values where that is allowed; viz., #2695)"""
+        q2 = np.array([[1., 2., 3.], [4., 5., 6.]]) * u.km / u.m
         q1 = q2.copy()
-        q2[0, 0] = 10000
+        q2[0, 0] = 10000.
         assert q2.unit == q1.unit
-        assert q2[0, 0].value == 10
-        q2[0] = 9 * u.Mm / u.km
-        assert all(q2.flatten()[:3].value == np.array([9, 9, 9]))
-        q2[0, :-1] = 8000
-        assert all(q2.flatten()[:3].value == np.array([8, 8, 9]))
+        assert q2[0, 0].value == 10.
+        q2[0] = 9. * u.Mm / u.km
+        assert all(q2.flatten()[:3].value == np.array([9., 9., 9.]))
+        q2[0, :-1] = 8000.
+        assert all(q2.flatten()[:3].value == np.array([8., 8., 9.]))
         with pytest.raises(u.UnitsError):
             q2[1, 1] = 10 * u.s
-        with pytest.raises(TypeError):
-            q2[0, 1] = 1.5 * u.km / u.m
+        # just to be sure, repeat with a dimensionfull unit
+        q3 = u.Quantity(np.arange(10.), "m/s")
+        q3[5] = 100. * u.cm / u.s
+        assert q3[5].value == 1.
+        # and check unit is ignored for 0, inf, nan, where that is reasonable
+        q3[5] = 0.
+        assert q3[5] == 0.
+        q3[5] = np.inf
+        assert np.isinf(q3[5])
+        q3[5] = np.nan
+        assert np.isnan(q3[5])
 
     def test_fill(self):
         q1 = np.array([1, 2, 3]) * u.m / u.km
@@ -399,6 +487,17 @@ class TestArrayConversion(object):
         assert q2.unit == q1.unit
         assert all(q2.value == q1.value.diagonal())
 
+    def test_view(self):
+        q1 = np.array([1, 2, 3], dtype=np.int64) * u.m / u.km
+        q2 = q1.view(np.ndarray)
+        assert not hasattr(q2, 'unit')
+        q3 = q2.view(u.Quantity)
+        assert q3._unit is None
+        # MaskedArray copies and properties assigned in __dict__
+        q4 = np.ma.MaskedArray(q1)
+        assert q4._unit is q1._unit
+        q5 = q4.view(u.Quantity)
+        assert q5.unit is q1.unit
 
     def test_slice_to_quantity(self):
         """
@@ -420,8 +519,6 @@ class TestArrayConversion(object):
         q2 = q1.astype(np.float64)
         assert all(q2 == q1)
         assert q2.dtype == np.float64
-        q2 = q1.view(np.ndarray)
-        assert not hasattr(q2, 'unit')
         q2a = q1.getfield(np.int32, offset=0)
         q2b = q1.byteswap().getfield(np.int32, offset=4)
         assert q2a.unit == q1.unit
@@ -444,7 +541,7 @@ class TestArrayConversion(object):
             q1.choose([0, 0, 1])
 
         with pytest.raises(NotImplementedError):
-            q1.list()
+            q1.tolist()
         with pytest.raises(NotImplementedError):
             q1.tostring()
         with pytest.raises(NotImplementedError):
@@ -453,3 +550,15 @@ class TestArrayConversion(object):
             q1.dump('a.a')
         with pytest.raises(NotImplementedError):
             q1.dumps()
+
+
+class TestRecArray(object):
+    """Record arrays are not specifically supported, but we should not
+    prevent their use unnecessarily"""
+    def test_creation(self):
+        ra = (np.array(np.arange(12.).reshape(4,3))
+              .view(dtype=('f8,f8,f8')).squeeze())
+        qra = u.Quantity(ra, u.m)
+        assert np.all(qra[:2].value == ra[:2])
+        qra[1] = qra[2]
+        assert qra[1] == qra[2]
