@@ -1043,7 +1043,7 @@ naxis kwarg.
                 xy = self._denormalize_sky(xy)
             output = func(xy, origin)
             if ra_dec_order and sky == 'output':
-                output = self._normalize_sky_output(output)
+                output = self._normalize_sky(output)
                 return (output[:, 0].reshape(axes[0].shape),
                         output[:, 1].reshape(axes[0].shape))
             return [output[:, i].reshape(axes[0].shape)
@@ -1997,6 +1997,12 @@ naxis kwarg.
         # (wcs[i] -> wcs.sub([i+1])
         return self.slice(item)
 
+    def __iter__(self):
+        # Having __getitem__ makes Python think WCS is iterable. However,
+        # Python first checks whether __iter__ is present, so we can raise an
+        # exception here.
+        raise TypeError("'{0}' object is not iterable".format(self.__class__.__name__))
+
     @property
     def axis_type_names(self):
         """
